@@ -9,25 +9,22 @@ import json
 
 
 class Transformer(object):
-    """模型基类
-    """
-    def __init__(
-        self,
-        vocab_size,  # 词表大小
-        hidden_size,  # 编码维度
-        num_hidden_layers,  # Transformer总层数
-        num_attention_heads,  # Attention的头数
-        intermediate_size,  # FeedForward的隐层维度
-        hidden_act,  # FeedForward隐层的激活函数
-        dropout_rate=None,  # Dropout比例
-        embedding_size=None,  # 是否指定embedding_size
-        attention_key_size=None,  # Attention中Q,K的head_size
-        sequence_length=None,  # 是否固定序列长度
-        keep_tokens=None,  # 要保留的词ID列表
-        layers=None,  # 外部传入的Keras层
-        name=None,  # 模型名称
-        **kwargs
-    ):
+    def __init__(self,
+                 vocab_size,
+                 hidden_size,
+                 num_hidden_layers,
+                 num_attention_heads,
+                 intermediate_size,
+                 hidden_act,
+                 dropout_rate=None,
+                 embedding_size=None,
+                 attention_key_size=None,
+                 sequence_length=None,
+                 keep_tokens=None,
+                 layers=None,
+                 name=None,
+                 **kwargs
+                 ):
         if keep_tokens is None:
             self.vocab_size = vocab_size
         else:
@@ -38,163 +35,126 @@ class Transformer(object):
         self.attention_head_size = hidden_size // num_attention_heads
         self.attention_key_size = attention_key_size or self.attention_head_size
         self.intermediate_size = intermediate_size
-        self.dropout_rate = dropout_rate or 0
         self.hidden_act = hidden_act
-        self.embedding_size = embedding_size or hidden_size
+        self.dropout_rate = dropout_rate or 0
+        self.embedding_size = embedding_size or self.hidden_size
         self.sequence_length = sequence_length
         self.keep_tokens = keep_tokens
         self.attention_mask = None
-        self.position_bias = None
+        self.position_bias=None,
         self.layers = {} if layers is None else layers
         self.name = name
         self.built = False
-
-    def build(
-        self,
-        layer_norm_cond=None,
-        layer_norm_cond_hidden_size=None,
-        layer_norm_cond_hidden_act=None,
-        additional_input_layers=None,
-        **kwargs
-    ):
-        """模型构建函数
-        layer_norm_*系列参数为实现Conditional Layer Normalization时使用，
-        用来实现以“固定长度向量”为条件的条件Bert。
-        """
+        
+    def build(self,
+              layer_norm_cond=None,
+              layer_norm_cond_hidden_size=None,
+              layer_norm_cond_hidden_act=None,
+              additional_input_layers=None,
+              **kwargs):
         if self.built:
             return None
-        # Input
+        
         inputs = self.get_inputs()
-        self.set_inputs(inputs, additional_input_layers)
-        # Other
+        self.set_inputs(inputs,additional_input_layers)
+        
         self.layer_norm_conds = [
-            layer_norm_cond,
-            layer_norm_cond_hidden_size,
-            layer_norm_cond_hidden_act or 'linear',
-        ]
-        # Call
+                                    layer_norm_cond,
+                                    layer_norm_cond_hidden_size,
+                                    layer_norm_cond_hidden_act or 'linear'
+                              ]
         outputs = self.call(inputs)
         self.set_outputs(outputs)
-        # Model
-        self.model = Model(self.inputs, self.outputs, name=self.name)
+        self.model = Model(self.inputs,self.outptus,name=self.name)
         self.built = True
-
-    def call(self, inputs):
-        """定义模型的执行流程
-        """
-        # Embedding
+    
+    def call(self,inputs):
         outputs = self.apply_embeddings(inputs)
-        # Main
         for i in range(self.num_hidden_layers):
-            outputs = self.apply_main_layers(outputs, i)
-        # Final
+            outputs = self.apply_main_layers(outputs,i)
         outputs = self.apply_final_layers(outputs)
         return outputs
-
-    def apply(self, inputs, layer=None, arguments=None, **kwargs):
-        """通过apply调用层会自动重用同名层
-        inputs: 上一层的输出；
-        layer: 要调用的层类名；
-        arguments: 传递给layer.call的参数；
-        kwargs: 传递给层初始化的参数。
-        """
-        if layer is Dropout and self.dropout_rate == 0:
-            return inputs
-
-        arguments = arguments or {}
-        name = kwargs.get('name')
-        if name not in self.layers:
-            layer = layer(**kwargs)
-            name = layer.name
-            self.layers[name] = layer
-
-        return self.layers[name](inputs, **arguments)
-
+    
     def get_inputs(self):
         raise NotImplementedError
-
-    def apply_embeddings(self, inputs):
-        raise NotImplementedError
-
-    def apply_main_layers(self, inputs, index):
-        raise NotImplementedError
-
-    def apply_final_layers(self, inputs):
-        raise NotImplementedError
-
-    def compute_attention_mask(self, inputs=None):
-        """定义每一层的Attention Mask
-        """
-        return self.attention_mask
-
-    def compute_position_bias(self, inputs=None):
-        """定义每一层的Position Bias（一般相对位置编码用）
-        """
-        return self.position_bias
-
-    def set_inputs(self, inputs, additional_input_layers=None):
-        """设置input和inputs属性
-        """
+    
+    def apply(self,inputs,layer=None,arguments=None,**kwargs):
+        if layer == Dropout and self.dropout_rate == 0:
+            return inputs
+        
+        arguments = arguments or {}
+        
+        name = kwargs['name']
+        if name not in self.layers:
+            layer = layer(**kwargs)
+            name=layer.name
+            self.layers[name] = layer
+            
+        return self.layers[name](inputs,**arguments)
+    
+    def set_inputs(self,inputs,additional_input_layers=None):
         if inputs is None:
             inputs = []
-        elif not isinstance(inputs, list):
+        elif not isinstance(inputs,list):
             inputs = [inputs]
-
+        
         inputs = inputs[:]
+        
         if additional_input_layers is not None:
-            if not isinstance(additional_input_layers, list):
+            if not isinstance(additional_input_layers,list):
                 additional_input_layers = [additional_input_layers]
             inputs.extend(additional_input_layers)
-
         self.inputs = inputs
         if len(inputs) > 1:
             self.input = inputs
         else:
             self.input = inputs[0]
-
-    def set_outputs(self, outputs):
-        """设置output和oututs属性
-        """
-        if not isinstance(outputs, list):
+    
+    def set_outputs(self,outputs):
+        if not isinstance(outputs,list):
             outputs = [outputs]
-
         outputs = outputs[:]
         self.outputs = outputs
         if len(outputs) > 1:
             self.output = outputs
         else:
             self.output = outputs[0]
-
+            
+    
+    def apply_embeddings(self,inputs):
+        raise NotImplementedError
+        
+    def apply_main_layers(self,inputs,index):
+        raise NotImplementedError
+    
+    def apply_final_layers(self,inputs):
+        raise NotImplementedError
+        
+    def compute_attention_mask(self,inputs=None):
+        return self.attention_mask
+    
+    def compute_position_bias(self,inputs=None):
+        return self.position_bias
+    
     @property
     def initializer(self):
-        """默认使用截断正态分布初始化
-        """
         return keras.initializers.TruncatedNormal(stddev=0.02)
-
-    def simplify(self, inputs):
-        """将list中的None过滤掉
-        """
+    
+    def simplify(self,inputs):
         inputs = [i for i in inputs if i is not None]
         if len(inputs) == 1:
             inputs = inputs[0]
-
         return inputs
-
-    def load_variable(self, checkpoint, name):
-        """加载单个变量的函数
-        """
-        return tf.train.load_variable(checkpoint, name)
-
-    def create_variable(self, name, value):
-        """在tensorflow中创建一个变量
-        """
-        return tf.Variable(value, name=name)
-
+    
+    def load_variable(self,checkpoint,name):
+        return tf.train.load_variable(checkpoint,name)
+    
+    def create_variable(self,name,value):
+        return tf.Variable(value,name=name)
+    
     def variable_mapping(self):
-        """构建keras层与checkpoint的变量名之间的映射表
-        """
         return {}
-
+    
     def load_weights_from_checkpoint(self, checkpoint, mapping=None):
         """根据mapping从checkpoint加载权重
         """
@@ -252,99 +212,99 @@ class Transformer(object):
                 saver.save(sess, filename, write_meta_graph=False)
 
 
+
 class BERT(Transformer):
-    """构建BERT模型
-    """
-    def __init__(
-        self,
-        max_position,  # 序列最大长度
-        with_pool=False,  # 是否包含Pool部分
-        with_nsp=False,  # 是否包含NSP部分
-        with_mlm=False,  # 是否包含MLM部分
-        custom_position_ids=False,  # 是否自行传入位置id
-        **kwargs  # 其余参数
-    ):
-        super(BERT, self).__init__(**kwargs)
+    def __init__(self,
+                 max_position,
+                 with_pool=False,
+                 with_nsp=False,
+                 with_mlm=False,
+                 custom_position_ids=False,
+                 **kwargs):
+        super(BERT,self).__init__(**kwargs)
         self.max_position = max_position
         self.with_pool = with_pool
         self.with_nsp = with_nsp
         self.with_mlm = with_mlm
         self.custom_position_ids = custom_position_ids
-
+    
     def get_inputs(self):
-        """BERT的输入是token_ids和segment_ids
-        （但允许自行传入位置id，以实现一些特殊需求）
-        """
-        x_in = Input(shape=(self.sequence_length,), name='Input-Token')
-        s_in = Input(shape=(self.sequence_length,), name='Input-Segment')
-
+        x_in = Input(shape=(self.sequence_length,),name='Input-Token')
+        s_in = Input(shape=(self.sequence_length,),name='Input-Segment')
         if self.custom_position_ids:
-            p_in = Input(shape=(self.sequence_length,), name='Input-Position')
-            return [x_in, s_in, p_in]
-        else:
-            return [x_in, s_in]
-
-    def apply_embeddings(self, inputs):
-        """BERT的embedding是token、position、segment三者embedding之和
-        """
-        x, s = inputs[:2]
+            p_in = Input(shape=(self.sequence_length,),name='Input-Position')
+            return [x_in,s_in,p_in]
+        return [x_in,s_in]
+    
+    def apply_embedding(self,inputs):
+        x,s = inputs[:2]
         z = self.layer_norm_conds[0]
         if self.custom_position_ids:
-            p = inputs[2]
+            p = input[2]
         else:
             p = None
+            
+        x = self.apply(
+                        inputs=x,
+                        layer=Embedding,
+                        input_dim=self.vocab_size,
+                        output_dim=self.embedding_size,
+                        embeddings_initializer=self.initializer,
+                        mask_zero=True,
+                        name='Embedding-Token'
+                       )
 
-        x = self.apply(
-            inputs=x,
-            layer=Embedding,
-            input_dim=self.vocab_size,
-            output_dim=self.embedding_size,
-            embeddings_initializer=self.initializer,
-            mask_zero=True,
-            name='Embedding-Token'
-        )
         s = self.apply(
-            inputs=s,
-            layer=Embedding,
-            input_dim=2,
-            output_dim=self.embedding_size,
-            embeddings_initializer=self.initializer,
-            name='Embedding-Segment'
-        )
-        x = self.apply(inputs=[x, s], layer=Add, name='Embedding-Token-Segment')
+                        inputs=s,
+                        layer=Embedding,
+                        input_dim=2,
+                        output_dim=self.embedding_size,
+                        embeddings_initializer=self.initializer,
+                        name='Embedding-Segment'
+                      )
+        
         x = self.apply(
-            inputs=self.simplify([x, p]),
-            layer=PositionEmbedding,
-            input_dim=self.max_position,
-            output_dim=self.embedding_size,
-            merge_mode='add',
-            embeddings_initializer=self.initializer,
-            custom_position_ids=self.custom_position_ids,
-            name='Embedding-Position'
-        )
+                        inputs=[x,s],
+                        layer=Add,
+                        name='Embedding-Token-Segment'
+                        )
+        
         x = self.apply(
-            inputs=self.simplify([x, z]),
-            layer=LayerNormalization,
-            conditional=(z is not None),
-            hidden_units=self.layer_norm_conds[1],
-            hidden_activation=self.layer_norm_conds[2],
-            hidden_initializer=self.initializer,
-            name='Embedding-Norm'
-        )
+                        inputs=self.simplify([x,p]),
+                        layer=PositionEmbedding,
+                        input_dim=self.max_position,
+                        output_dim=self.embedding_size,
+                        merge_mode='add',
+                        embedding_initializer=self.initializer,
+                        custom_position_ids=self.custom_position_ids,
+                        name='Embedding-Position'
+                     )
+        
         x = self.apply(
-            inputs=x,
-            layer=Dropout,
-            rate=self.dropout_rate,
-            name='Embedding-Dropout'
-        )
+                        inputs=self.simplify([x,z]),
+                        layer=LayerNormalization,
+                        conditinal=(z is not None),
+                        hidden_units=self.layer_norm_conds[1],
+                        hidden_activation=self.layer_norm_conds[2],
+                        hidden_initializer=self.initializer,
+                        name='Embedding-Norm'
+                     )
+        
+        x = self.apply(
+                        inputs=x,
+                        layer=Dropout,
+                        rate=self.dropout_rate,
+                        name='Embedding-Dropout'
+                      )
+        
         if self.embedding_size != self.hidden_size:
             x = self.apply(
-                inputs=x,
-                layer=Dense,
-                units=self.hidden_size,
-                kernel_initializer=self.initializer,
-                name='Embedding-Mapping'
-            )
+                            inputs=x,
+                            layer=Dense,
+                            units=self.hidden_size,
+                            kernel_initializer=self.initializer,
+                            name='Embedding-Mapping'
+                          )
 
         return x
 
@@ -597,8 +557,6 @@ class BERT(Transformer):
         return mapping
 
 
-
-
 def extend_with_language_model(BaseModel):
     """添加下三角的Attention Mask（语言模型用）
     """
@@ -633,86 +591,6 @@ def extend_with_language_model(BaseModel):
     return LanguageModel
 
 
-def extend_with_unified_language_model(BaseModel):
-    """添加UniLM的Attention Mask（UnifiedLanguageModel用）
-    """
-    class UnifiedLanguageModel(BaseModel):
-        """带UniLM的Attention Mask的派生模型
-        UniLM: https://arxiv.org/abs/1905.03197
-        """
-        def __init__(self, *args, **kwargs):
-            super(UnifiedLanguageModel, self).__init__(*args, **kwargs)
-            self.with_mlm = self.with_mlm or True
-
-        def compute_attention_mask(self, inputs=None):
-            """重载此函数即可
-            """
-            if self.attention_mask is None:
-
-                def unilm_mask(s):
-                    idxs = K.cumsum(s, axis=1)
-                    mask = idxs[:, None, :] <= idxs[:, :, None]
-                    mask = K.cast(mask, K.floatx())
-                    return mask[:, None]
-
-                self.attention_mask = self.apply(
-                    inputs=self.inputs[1],
-                    layer=Lambda,
-                    function=unilm_mask,
-                    name='Attention-UniLM-Mask'
-                )
-
-            return self.attention_mask
-
-    return UnifiedLanguageModel
-
-
-def extend_with_parallel_unified_language_model(BaseModel):
-    """添加并行式UniLM的Attention Mask（Seq2Seq用）
-    注：用于加速一对多的Seq2Seq任务的训练过程。
-    """
-    class ParallelUnifiedLanguageModel(BaseModel):
-        """带并行式UniLM的Attention Mask的派生模型
-        UniLM: https://arxiv.org/abs/1905.03197
-        """
-        def __init__(self, *args, **kwargs):
-            super(ParallelUnifiedLanguageModel, self).__init__(*args, **kwargs)
-            self.with_mlm = self.with_mlm or True
-            self.custom_position_ids = True
-
-        def compute_attention_mask(self, inputs=None):
-            """重载此函数即可
-            """
-            if self.attention_mask is None:
-
-                def punilm_mask(inputs):
-                    s, p = inputs
-
-                    s_idxs = K.cumsum(s, axis=1)
-                    s_mask = s_idxs[:, None, :] <= s_idxs[:, :, None]
-                    s_mask = K.cast(s_mask, K.floatx())
-
-                    p = K.less(p[:, 1:] - p[:, :-1], 0)
-                    p = K.cast(p, K.floatx())
-                    p = K.concatenate([K.zeros_like(p[:, :1]), p], axis=1)
-                    p_idxs = K.cumsum(p, axis=1) + s
-                    p_mask = K.equal(p_idxs[:, None, :], p_idxs[:, :, None])
-                    p_mask = K.cast(p_mask, K.floatx())
-
-                    s = s[:, None]
-                    mask = s_mask * (1 - s) + p_mask * s_mask * s
-                    return mask[:, None]
-
-                self.attention_mask = self.apply(
-                    inputs=self.inputs[1:3],
-                    layer=Lambda,
-                    function=punilm_mask,
-                    name='Attention-PUniLM-Mask'
-                )
-
-            return self.attention_mask
-
-    return ParallelUnifiedLanguageModel
 
 
 def build_transformer_model(
@@ -737,7 +615,13 @@ def build_transformer_model(
     model, application = model.lower(), application.lower()
 
     models = {
-        'bert': BERT
+        'bert': BERT,
+        'albert': ALBERT,
+        'albert_unshared': ALBERT_Unshared,
+        'nezha': NEZHA,
+        'electra': ELECTRA,
+        'gpt2_ml': GPT2_ML,
+        't5': T5,
     }
     MODEL = models[model]
 
